import 'package:chanworker/chanworker.dart';

/// AggregatingEcho Service is not very special, it simply remembers all that it has heard before,
/// each time it hears something it concatenates with what it has heard before and echos it back
/// after a delay. To simulate the service doing real work we recursively calculate the fib number
/// related to the call number.
class AggregatingEcho {
  String _rememberedFromBefore = '';
  int _calls = 0;

  final Channel consumerChannel;

  AggregatingEcho(this.consumerChannel) {
    consumerChannel.receivePort.listen(_handleIncomingMessage);
  }

  /// Handles messages sent to this service.
  _handleIncomingMessage(m) {
    print('Service: call with: $m of type: ${m.runtimeType}');
    _calls++;
    DateTime start = DateTime.now();
    print('-> Calculating the ${_calls.toString()} Fibonacci number.');
    int result = _calFibForCall(_calls);
    print(
        '-> Answer: $result, that took ${DateTime.now().difference(start).inMilliseconds} Milliseconds.');
    _rememberedFromBefore += m + ' ' as String;
    consumerChannel.dispatchPort.add(_rememberedFromBefore);
  }

  int _calFibForCall(int callCount) {
    if (callCount < 2) {
      return callCount;
    } else {
      return _calFibForCall(callCount - 2) + _calFibForCall(callCount - 1);
    }
  }
}

/// This only gets invoked that [AggregatingEcho] service is loaded by a [Worker]
main() {
  AggregatingEcho(WorkerChannelCounterPart<String>());
}
